#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <asm/types.h>
#include <linux/netlink.h>
#include <linux/socket.h>
#include <signal.h>
#include <pthread.h>
#include <kysec/kysec_communicate.h>
#include <kysec/kysec_xml.h>

#define MAX_PAYLOAD		1024	/*消息最大负载为1024字节*/
#define NETLINK_KYSEC_KMOD	27

#define NETLINK_KYSEC_GROUP_HASH 2
#define NETLINK_KYSEC_HASH 27
#define KYSEC_HASH_MESG_MAX 100

int kysec_kmod_communicate_with_kernel(char *command)
{
	struct sockaddr_nl src_addr, dest_addr;
	struct nlmsghdr *nlh = NULL;
	int sock_fd = -1;
	int retval = 0;
	int state = 0;
	int dest_addr_len = 0;
	char reply_msg[MAX_PAYLOAD + 1] = {0};
	char *ptr = NULL;
	int op_type;
	//	int data_len = 0;

	memset(&src_addr, 0, sizeof(src_addr));
	src_addr.nl_family = AF_NETLINK;
	src_addr.nl_pid = getpid();
	src_addr.nl_groups = 0;

	memset(&dest_addr, 0, sizeof(dest_addr));
	dest_addr.nl_family = AF_NETLINK;
	dest_addr.nl_pid = 0;
	dest_addr.nl_groups = 0;

	sock_fd = socket(AF_NETLINK, SOCK_RAW, NETLINK_KYSEC_KMOD);
	if (-1 == sock_fd) {
		perror("can't create netlink socket!");
		return -1;
	}

	retval = bind(sock_fd, (struct sockaddr *)&src_addr, sizeof(src_addr));
	if (retval < 0) {
		perror("can't bind sockfd with sockaddr_nl!");
		close(sock_fd);
		return -1;
	}

	if (NULL == (nlh = (struct nlmsghdr *)malloc(NLMSG_SPACE(MAX_PAYLOAD)))) {
		perror("alloc mem failed!");
		close(sock_fd);
		return -1;
	}

	if(!strncmp("del", command, 3)){
		op_type = 8;
	}else if(!strncmp("add", command, 3)){
		op_type = 7;
	}else{
		return -1;
	}

	memset(nlh, 0, NLMSG_SPACE(MAX_PAYLOAD));
	/* 填充Netlink消息头部 */
	nlh->nlmsg_len = NLMSG_SPACE(MAX_PAYLOAD);
	nlh->nlmsg_pid = getpid();
	nlh->nlmsg_flags = 0;
	nlh->nlmsg_type = op_type;
	strncpy(NLMSG_DATA(nlh), command+4, strlen(command) - 3);

	state = sendto(sock_fd, (void *)nlh, NLMSG_SPACE(MAX_PAYLOAD), 0,
			(struct sockaddr *)(&dest_addr), sizeof(dest_addr));
	if (state < 0) {
		perror("tkl sendto failed!");
		retval = -1;
		goto out;
	}

	//接收内核消息的消息
	memset(nlh, 0, NLMSG_SPACE(MAX_PAYLOAD)); //清空整个Netlink消息头包括消息头和负载
	dest_addr_len = sizeof(dest_addr);
	state = recvfrom(sock_fd, (void *)nlh, NLMSG_SPACE(MAX_PAYLOAD), 0,
			(struct sockaddr *)(&dest_addr), (socklen_t *)&dest_addr_len);
	if (state < 0) {
		perror("recvfrom failed!");
		retval = -1;
		goto out;
	}

	ptr = NLMSG_DATA(nlh);
	for (int i = 0; i < MAX_PAYLOAD && i < state; i++) {
		if (*ptr == '\0')
			break;
		reply_msg[i] = *ptr++;
	}

	if (strcmp(reply_msg, "succeeded") != 0)
		printf("Got response: %s\n", reply_msg);
	retval = 0;

out:
	close(sock_fd);
	free(nlh);

	return retval;
}

int kysec_convert_exectl_label_to_label_type(char *exectl_label)
{
	if (strcmp(exectl_label, "verified") == 0)
		return EXECTL_VERIFIED_TYPE;
	else if (strcmp(exectl_label, "original") == 0)
		return EXECTL_ORIGINAL_TYPE;
	else if (strcmp(exectl_label, "kysoft") == 0)
		return EXECTL_KYSOFT_TYPE;
	else if (strcmp(exectl_label, "trusted") == 0)
		return EXECTL_TRUSTED_TYPE;
	else
		return EXECTL_UNKNOWN_TYPE;
}

static int kysec_communicate_with_kernel(int mesg_type, void *data, int data_len)
{
	struct sockaddr_nl src_addr, dest_addr;
	struct nlmsghdr *nlh = NULL;
	int sock_fd = -1;
	int retval = 0, state;
	int dest_addr_len;
	char reply_msg[KYSEC_HASH_MESG_MAX + 1] = {0}, *ptr;

	if (!data) {
		printf("kysec-notify: data was wrong\n");
		return -1;
	}

	memset(&src_addr, 0, sizeof(src_addr));
	src_addr.nl_family = AF_NETLINK;
	src_addr.nl_pid = getpid();
	src_addr.nl_groups = 0;

	memset(&dest_addr, 0, sizeof(dest_addr));
	dest_addr.nl_family = AF_NETLINK;
	dest_addr.nl_pid = 0;
	dest_addr.nl_groups = 0;

	sock_fd = socket(AF_NETLINK, SOCK_RAW, NETLINK_KYSEC_HASH);
	if (-1 == sock_fd) {
		perror("can't create netlink socket!");
		return -1;
	}

	retval = bind(sock_fd, (struct sockaddr *)&src_addr, sizeof(src_addr));
	if (retval < 0) {
		perror("can't bind sockfd with sockaddr_nl!");
		close(sock_fd);
		return -1;
	}

	if (NULL == (nlh = (struct nlmsghdr *)malloc(NLMSG_SPACE(KYSEC_HASH_MESG_MAX)))) {
		perror("alloc mem failed!");
		close(sock_fd);
		return -1;
	}

	memset(nlh, 0, NLMSG_SPACE(KYSEC_HASH_MESG_MAX));
	/* 填充Netlink消息头部 */
	nlh->nlmsg_len = NLMSG_SPACE(KYSEC_HASH_MESG_MAX);
	nlh->nlmsg_pid = getpid();
	nlh->nlmsg_flags = 0;
	nlh->nlmsg_type = mesg_type;

	memcpy(NLMSG_DATA(nlh), data, data_len);

	state = sendto(sock_fd, (void *)nlh, NLMSG_SPACE(KYSEC_HASH_MESG_MAX),
			0, (struct sockaddr *)(&dest_addr), sizeof(dest_addr));
	if (state < 0) {
		perror("kysec-notify: sendto failed!");
		retval = -1;
		goto out;
	}

	memset(nlh, 0, NLMSG_SPACE(KYSEC_HASH_MESG_MAX)); //清空整个Netlink消息头包括消息头和负载
	dest_addr_len = sizeof(dest_addr);
	state = recvfrom(sock_fd, (void *)nlh, NLMSG_SPACE(KYSEC_HASH_MESG_MAX),
			0, (struct sockaddr *)(&dest_addr), (socklen_t *)&dest_addr_len);
	if (state < 0) {
		perror("kysec-notify: recvfrom failed!");
		retval = -1;
		goto out;
	}

	ptr = NLMSG_DATA(nlh);
	for (int i = 0; i < KYSEC_HASH_MESG_MAX && i < state; i++) {
		if (*ptr == '\0')
			break;
		reply_msg[i] = *ptr++;
	}

	if (strcmp(reply_msg, "succeeded") != 0)
		printf("Got response: %s\n", reply_msg);
	retval = 0;
out:
	close(sock_fd);
	free(nlh);

	return retval;
}

struct kysec_label_info {
	int label_type;
	char hash[65];    /* 64 + 1 */
};

int kysec_hash_communicate_with_kernel(int mesg_type, char *hash, int label_type)
{
	struct kysec_label_info info;

	if (!hash)
		return -1;

	memset(&info, 0, sizeof(info));
	info.label_type = label_type;
	memcpy(info.hash, hash, strlen(hash));

	return kysec_communicate_with_kernel(mesg_type, &info, sizeof(info));
}

struct kysec_netctl_info {
	int op_type;
	char hash[XML_HASH_SIZE + 1];
};

int kysec_netctl_communicate_with_kernel(int mesg_type, char *hash, int netctl_type)
{
	struct kysec_netctl_info info;

	if (!hash)
		return -1;

	memset(&info, 0, sizeof(info));
	info.op_type = netctl_type;
	memcpy(info.hash, hash, strlen(hash));

	return kysec_communicate_with_kernel(mesg_type, &info, sizeof(info));
}
