/*
 *  Kylin security execute control libraries
 *
 *  This file contains the Kylin security exectl status functions
 *  implementations.
 *
 *  Authors:  songchuanfei <songchuanfei@kylinos.cn>
 *  Authors:  huanfuxi <huanfuxi@kylinos.cn>
 *  Authors:  mengyuan <mengyuan@kylinos.cn>
 *
 *  Copyright (C) 2017,Tianjin KYLIN Information Technology Co., Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */
#include <sys/types.h>
#include <stdio.h>
#include <mntent.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <kysec/status.h>

#ifndef PATH_MAX
#define PATH_MAX 255
#endif

static int securityfs_mounted(const char *mount_point)
{
	int ret = 0;
	struct mntent *mnt = NULL;
	FILE *mtab = setmntent("/proc/mounts", "r");

	if (!mtab)
		return -1;

	while ((mnt = getmntent(mtab)) != NULL) {
		if (!strcmp(mnt->mnt_dir, mount_point) && !strcmp(mnt->mnt_type, "securityfs")) {
			ret = 1;
			break;
		}
	}

	endmntent(mtab);

	return ret;
}

/* The function to get the kysec status
 * return following values:
 *  -1 : get the kysec_softmode file error
 *   0 : the Kylin Security is disabled
 *   2 : the Kylin Security is Normal
 *   4 : the Kylin Seucrity is softmode
 *   5 : the Kylin Seucrity is warning
 */
int kysec_getstatus(void)
{
	int fd, ret, state = 0;
	char path[PATH_MAX];
	char buf[20];

	fd = open(KYSEC_STATUS_PATH, O_RDONLY);
	if (fd < 0) {
		if (errno == ENOENT) {
			int ret = securityfs_mounted(SECURITYFS_MNT);
			if (ret <= 0) {
				//fprintf(stderr, "securityfs not mounted, mount it and try again, ret: %d\n", ret);
				errno = ENOENT;
				return 0;
			} else
				return 0;
		}
		goto out;
	}
	memset(buf, 0, sizeof(buf));
	ret = read(fd, buf, sizeof(buf - 1));
	close(fd);

	if (ret < 0)
		goto out;

	if (sscanf(buf, "%d", &state) != 1)
		goto out;

	return state;

out:
	return -1;
}

int kysec_get_func_status(int func_id)
{
	int fd, ret, state;
	char buf[20];

	if (func_id < 0 || func_id >= FUNC_MAX) {
		errno = EINVAL;
		return -1;
	}

	fd = open(func_entries[func_id], O_RDONLY);
	if (fd < 0) {
		if (errno == ENOENT) {
			int ret = securityfs_mounted(SECURITYFS_MNT);

			if (ret <= 0) {
				//fprintf(stderr, "securityfs not mounted, mount it and try again, ret: %d\n", ret);
				errno = ENOENT;
				return 0;
			} else
				return 0;
		}
		goto out;
	}

	memset(buf, 0, sizeof(buf));
	ret = read(fd, buf, sizeof(buf - 1));
	close(fd);

	if (ret < 0)
		goto out;

	if (sscanf(buf, "%d", &state) != 1)
		goto out;

	return state;

out:
	return -1;
}
