/*
 *  Kylin security execute control libraries
 *
 *  This file contains the Kylin security exectl status functions
 *  implementations.
 *
 *  Authors:  songchuanfei <songchuanfei@kylinos.cn>
 *  Authors:  huanfuxi <huanfuxi@kylinos.cn>
 *  Authors:  mengyuan <mengyuan@kylinos.cn>
 *
 *  Copyright (C) 2017,Tianjin KYLIN Information Technology Co., Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */
#include <sys/types.h>
#include <stdio.h>
#include <mntent.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <kysec/status_dbus.h>
#include <glib.h>
#include <gio/gio.h>

#ifndef PATH_MAX
#define PATH_MAX 255
#endif

extern int handling_dbus_error(GError *error);
extern GVariant *process_dbus_method(gchar *method, GVariant *parameters, int *err_code);

/* The function is to set the KySec status
 * return following values:
 *  -2 : the Kylin Security disabled
 *  -1 : set kysec_softmode error
 *   0 : set kysec_softmode success
 */
int kysec_setstatus(int value)
{
	int err_code;
	GVariant *parameters = g_variant_new("(i)", value);
	GVariant *result = process_dbus_method("kysec_xattr_set_status", parameters, &err_code);

	if (NULL == result)
		return err_code;

	int ret = g_variant_get_int32(g_variant_get_child_value(result, 0));

	g_variant_unref(result);

	return ret;
}

int kysec_set_func_status(int func_id, int status)
{
	int err_code;
	GVariant *parameters = g_variant_new("(ii)", func_id, status);
	GVariant* result = process_dbus_method("kysec_xattr_set_func_status", parameters, &err_code);

	if (NULL == result)
		return err_code;

	int ret = g_variant_get_int32(g_variant_get_child_value(result, 0));

	g_variant_unref(result);

	return ret;
}
