#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <gelf.h>
#include <libelf.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "kysecwhlist/kysec_file_type.h"
#include <syslog.h>

static int endwith(char *src, char *str)
{
	char *p = NULL;

	p = strrchr(src, '.');
	if (p != NULL && strcmp(p, str) == 0)
		return 1;
	else
		return 0;
}

static int is_shell(const char *filename)
{
#define BUF_LEN 20

	char buf[BUF_LEN] = {0};
	FILE *fp = NULL;
	int len = 0;
	int i = 0, j = 0;
	int ret = 0;

	fp = fopen(filename,"r");
	if (fp == NULL)
		return -1;

	if (fgets(buf, BUF_LEN, fp) == NULL) {
		fclose(fp);
		return -1;
	}

	fclose(fp);
	len = strlen(buf);

	for (i = 0; i < len; i++) {
		if (buf[i] != ' '){
			buf[j] = buf[i];
			j++;
		}
		if (j > 6)
			break;
	}

	buf[j] = '\0';

	if (strcmp(buf, "#!/bin/") == 0)
		ret = 1;

	return ret;

#undef BUF_LEN
}

static int is_python(const char *filename)
{
#define BUF_LEN 20

	char buf[BUF_LEN] = {0};
	FILE *fp = NULL;
	int i = 0, j = 0;

	fp = fopen(filename,"r");
	if (fp == NULL)
		return -1;

	if (fgets(buf, BUF_LEN, fp) == NULL) {
		fclose(fp);
		return -1;
	}

	fclose(fp);

	for (i = 0, j = 0; buf[i]; i++) {
		if (buf[i] == '\n')
			buf[i] = '\0';
		if (buf[i] != ' ')
			buf[j++] = buf[i];
	}

	buf[j] = '\0';

	if ((strstr(filename, ".py") != NULL && strstr(filename, ".pyc") == NULL) ||
	    (strstr(buf, "python") != NULL && strstr(buf, "#!/") != NULL))
		return 1;
	else
		return 0;

#undef BUF_LEN
}

int parse_file_type(const char *filename)
{
	int ret = -1;
	int fd = -1;
	Elf *elf = NULL;
	GElf_Ehdr elfhdr;
	struct stat sb;
	char file_name[128]={0};

	strncpy(file_name, filename, strlen(filename));
	if (lstat(file_name, &sb) == -1)
		return -1;

	switch (sb.st_mode & S_IFMT) {
//	case S_IFBLK:
//	case S_IFCHR:
//	case S_IFIFO:
//	case S_IFSOCK:
//	case S_IFDIR:
//	case S_IFLNK:
//		return -1;
	case S_IFLNK:
	case S_IFREG:
		break;
	default:
		return -1;
	}

	if (is_shell(file_name) == 1 || is_python(file_name) == 1)
		return object_script;
	        
	if (NULL != strstr(file_name, ".xz")) {
		char kmod_cmd[256]={0};
		sprintf(kmod_cmd, "mkdir /tmp/kmod_dir;cp %s /tmp/kmod_dir/;xz -d /tmp/kmod_dir/*;realpath /tmp/kmod_dir/* > /tmp/kmod_dir/filename", file_name);
		system(kmod_cmd);
		FILE *fp = fopen("/tmp/kmod_dir/filename","r");
		memset(file_name,0,128);
		fgets(file_name,128,fp);
		file_name[strlen(file_name)-1] = 0;
		fclose(fp);
	}

	fd = open(file_name, O_RDONLY);
	if (fd < 0)
		return -1;

	elf_version(EV_CURRENT);
	elf = elf_begin(fd, ELF_C_READ, NULL);
	close(fd);
	if (!elf) {
		/* 解析Descriptor for the ELF file返回为0，
		 * 表示该文件非ELF文件，暂将该文件定为其他类型文件
		 */
		return object_other;
	}

	if (gelf_getehdr(elf, &elfhdr) == NULL) {
		/* 解析ELF文件头返回为0，表示该文件非ELF文件，
		 * 暂将该文件定为其他类型文件
		 */
		elf_end(elf);
		return object_other;
	}

	system("rm -rf /tmp/kmod_dir");
	switch (elfhdr.e_type) {
	case ET_DYN: {
		char* fname = strrchr(file_name, '/');
		if (fname == NULL) {
			if(strstr(file_name, ".so") == NULL && strstr(file_name, ".so.") == NULL)
				ret = object_program;
			else
				ret = object_library;
		} else {
			if(strstr(fname, ".so") == NULL && strstr(fname, ".so.") == NULL)
				ret = object_program;
			else
				ret = object_library;
		}
		break;
	}
	case ET_REL:
		/* 重定向文件且后缀为.ko，该文件为内核模块文件 */
		if (endwith((char *)file_name, ".ko"))
			ret = object_kmodule;
		else
			ret = object_other;
		break;
	case ET_EXEC:
		ret = object_program;
		break;
	default:
		ret = object_other;
		break;
	}

	elf_end(elf);

	return ret;
}
