#include <kysecwhlist/kysec_whlist.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <stdio.h>
#include <fcntl.h>
#include <glib.h>
#include <gio/gio.h>
#include <locale.h>
#include <syslog.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <kysec/kysec_communicate.h>
#include <kysec/kmod_protect.h>
#include <syslog.h>

//白名单数据库记录
struct db_object_inner {
	char *fpath;	//文件路径
	char *hash;	//摘要值
	char *type;	//文件类型
	char *exectl;	//执行控制标记类型
	char *filepro;	//文件保护标记类型
	int rmmodpro;	//是否内核防卸载
	int status;	//数据项状态
};

/**
 * @brief 单链表结构体定义
 *
 * 单链表结构体名字是object_list，链表成员类型是object_entry
 */
SLIST_HEAD(object_list_inner, object_entry_inner);
struct object_entry_inner {
	struct db_object_inner object;
	SLIST_ENTRY(object_entry_inner)
	entries;
};

#define SYSTEM_DBUS_SERVICE "com.kylin.kysec"
#define SYSTEM_OBJECT_PATH "/whlist"
#define SYSTEM_DBUS_INTERFACE "com.kylin.kysec.whlist"
#define SYSTEM_DBUS_TIMEOUT 5000

#define KYSEC_WHLIST_METHOD_ADD "kysec_whlist_add"
#define KYSEC_WHLIST_METHOD_DEL "kysec_whlist_del"
#define KYSEC_WHLIST_METHOD_UPDATE "kysec_whlist_update"
#define KYSEC_WHLIST_METHOD_UPDATE_RMMODPRO "kysec_whlist_update_rmmodpro"
#define KYSEC_WHLIST_METHOD_LOAD "kysec_whlist_load"
#define KYSEC_WHLIST_METHOD_IMPORT "kysec_whlist_import"
#define KYSEC_WHLIST_METHOD_EXPORT "kysec_whlist_export"

#define KYSEC_WHLIST_METHOD_GET_COUNT_BY_OBJECT_TYPE "kysec_whlist_get_count_by_obj_type"
#define KYSEC_WHLIST_METHOD_LOAD_BY_OBJECT_TYPE "kysec_whlist_load_by_obj_type"
#define KYSEC_WHLIST_METHOD_LOAD_BY_PATH "kysec_whlist_load_by_path"
#define KYSEC_WHLIST_METHOD_ADD_BY_DEB "kysec_whlist_add_by_deb"
#define KYSEC_WHLIST_METHOD_UPDATE_BY_DEB "kysec_whlist_update_by_deb"
#define KYSEC_WHLIST_METHOD_DEL_BY_DEB "kysec_whlist_del_by_deb"
#define KYSEC_WHLIST_METHOD_ADD_BY_DIR "kysec_whlist_add_by_dir"
#define KYSEC_WHLIST_METHOD_UPDATE_BY_DIR "kysec_whlist_update_by_dir"
#define KYSEC_WHLIST_METHOD_DEL_BY_DIR "kysec_whlist_del_by_dir"

#define KYSEC_WHLIST_METHOD_GET_HASH_BY_PATH "kysec_whlist_get_hash_by_path"

#define MAX_ERR_MSG_LEN 1024

static char err_msg[MAX_ERR_MSG_LEN];
#define TMP_DIR "/tmp/"

static void kysec_whlist_do_syslog(const char *format, ...)
{
	va_list vlist;

	va_start(vlist, format);

	openlog("KYSEC(LIB)", 0, LOG_LOCAL0);
	vsyslog(LOG_WARNING, format, vlist);
	closelog();

	va_end(vlist);
}

static int kysec_whlist_handling_dbus_error(GError *error)
{
	int retval = 0;
	gchar *mesg = NULL;
	gchar *errName = NULL;

	if (g_dbus_error_is_remote_error(error)) {
		errName = g_dbus_error_get_remote_error(error);
		g_dbus_error_strip_remote_error(error);

		mesg = error->message;
		if (mesg) {
			kysec_whlist_do_syslog("%s: %s\n", errName, mesg);
			memset(err_msg, 0x00, sizeof(err_msg));
			snprintf(err_msg, sizeof(err_msg), "%s", mesg);
		}
		
		if (strcmp(errName, "KYLIN.KYSEC.OK") == 0)
			retval = 0;
		else if (strcmp(errName, "KYLIN.KYSEC.AUTH.FAILED") == 0)
			retval = 1;
		else if (strcmp(errName, "KYLIN.KYSEC.CANCEL.AUTH") == 0)
			retval = 2;
		else if (strcmp(errName, "KYLIN.KYSEC.DISABLED") == 0)
			retval = 3;
		else if (strcmp(errName, "KYLIN.KYSEC.DATABASE.ERR") == 0)
			retval = 4;
		else if (strcmp(errName, "KYLIN.KYSEC.LABEL.GET.ERR") == 0)
			retval = 5;
		else if (strcmp(errName, "KYLIN.KYSEC.LABEL.SET.ERR") == 0)
			retval = 6;
		else if (strcmp(errName, "KYLIN.KYSEC.INVALID.ARG") == 0)
			retval = 7;
		else if (strcmp(errName, "KYLIN.KYSEC.EXTRACT.LIST.ERR") == 0)
			retval = 8;
		else if (strcmp(errName, "KYLIN.KYSEC.FILE.ATTR.ERR") == 0)
			retval = 9;
		else 
			retval = 10;		
	}

	return retval;
}

static GVariant *kysec_whlist_process_dbus_method(gchar *method, GVariant *parameters, int *err_code)
{
	GError *error = NULL;
	GDBusProxy *proxy = NULL;
	GVariant *result = NULL;

	proxy = g_dbus_proxy_new_for_bus_sync(
				G_BUS_TYPE_SYSTEM,
				G_DBUS_PROXY_FLAGS_NONE,
				NULL,
				SYSTEM_DBUS_SERVICE,
				SYSTEM_OBJECT_PATH,
				SYSTEM_DBUS_INTERFACE,
				NULL,
				&error);

	if (error) {
		*err_code = kysec_whlist_handling_dbus_error(error);
		g_error_free(error);
		error = NULL;
		return NULL;
	}

	result = g_dbus_proxy_call_sync(
				proxy,
				method,
				parameters,
				G_DBUS_CALL_FLAGS_NONE,
				G_MAXINT,
				NULL,
				&error);

	if (error) {
		*err_code = kysec_whlist_handling_dbus_error(error);
		g_error_free(error);
		error = NULL;
		g_object_unref(proxy);
		return NULL;
	}

	g_object_unref(proxy);
	return result;
}

static int parse_exectl_type(char *exectl)
{
	if (strcmp(exectl, "original") == 0)
		return exectl_sign_original;
	else if (strcmp(exectl, "verified") == 0)
		return exectl_sign_verified;
	else if (strcmp(exectl, "kysoft") == 0)
		return exectl_sign_kysoft;
	else if (strcmp(exectl, "trusted") == 0)
		return exectl_sign_trusted;
	else
		return exectl_sign_unknown;
}

static int parse_protect_type(char *filepro)
{
	if (strcmp(filepro, "readonly") == 0)
		return 0;
	else
		return 1;
}

static int parse_kysec_file_type(char *type)
{
	if (strcmp(type, "program") == 0)
		return 0;
	else if (strcmp(type, "library") == 0)
		return 1;
	else if (strcmp(type, "script") == 0)
		return 2;
	else if (strcmp(type, "kmodule") == 0)
		return 3;
	else
		return 4;
}

static int create_object_list_entry(struct object_list *objects,
				    const char *fpath, const char *hash,
				    char *type, char *exectl, char *filepro,
				    int rmmodpro, int status)
{
	struct object_entry *entry = NULL;

	entry = (struct object_entry *)calloc(sizeof(struct object_entry), 1);
	if (!entry) {
		kysec_whlist_do_syslog("%s: failed to malloc\n", __func__);
		snprintf(err_msg, sizeof(err_msg), "%s: the parameter is NULL\n", __func__);
		return -1;
	}

	if (fpath)
		entry->object.fpath = strndup(fpath, strlen(fpath));
	else
		entry->object.fpath = NULL;

	if (hash)
		entry->object.hash = strndup(hash, strlen(hash));
	else
		entry->object.hash = NULL;
	entry->object.type = parse_kysec_file_type(type);
	entry->object.exectl = parse_exectl_type(exectl);
	entry->object.filepro = parse_protect_type(filepro);
	entry->object.rmmodpro = rmmodpro;
	entry->object.status = status;

	SLIST_INSERT_HEAD(objects, entry, entries);

	return 0;
}

static int create_object_list_entry_inner(struct object_list_inner *objects,
					  const char *fpath, const char *hash,
					  char *type, char *exectl, char *filepro,
					  int rmmodpro, int status)
{
	struct object_entry_inner *entry = NULL;

	entry = (struct object_entry_inner *)calloc(sizeof(struct object_entry_inner), 1);
	if (!entry) {
		kysec_whlist_do_syslog("%s: failed to malloc\n", __func__);
		snprintf(err_msg, sizeof(err_msg), "%s: the parameter is NULL\n", __func__);
		return -1;
	}

	if (fpath)
		entry->object.fpath = strndup(fpath, strlen(fpath));
	else
		entry->object.fpath = NULL;
	if (hash)
		entry->object.hash = strndup(hash, strlen(hash));
	else
		entry->object.hash = NULL;
	if (type)
		entry->object.type = strndup(type, strlen(type));
	else
		entry->object.type = NULL;
	if (exectl)
		entry->object.exectl = strndup(exectl, strlen(exectl));
	else
		entry->object.exectl = NULL;
	if (filepro)
		entry->object.filepro = strndup(filepro, strlen(filepro));
	else
		entry->object.filepro = NULL;
	entry->object.rmmodpro = rmmodpro;
	entry->object.status = status;

	SLIST_INSERT_HEAD(objects, entry, entries);

	return 0;
}

static void kysec_whlist_objects_inner_free(struct object_list_inner *objects)
{
	struct object_entry_inner *entry;

	if (!objects)
		return;

	entry = SLIST_FIRST(objects);
	for (; entry != NULL; entry = SLIST_FIRST(objects)) {
		SLIST_REMOVE_HEAD(objects, entries);
		if (entry->object.fpath) {
			free(entry->object.fpath);
			entry->object.fpath = NULL;
		}

		if (entry->object.hash) {
			free(entry->object.hash);
			entry->object.hash = NULL;
		}

		if (entry->object.type) {
			free(entry->object.type);
			entry->object.type = NULL;
		}

		if (entry->object.exectl) {
			free(entry->object.exectl);
			entry->object.exectl = NULL;
		}

		if (entry->object.filepro) {
			free(entry->object.filepro);
			entry->object.filepro = NULL;
		}

		if (entry) {
			free(entry);
			entry = NULL;
		}
	}

	return;
}

//根据功能类型获取白名单（kysec_type： 0 执行控制； 1 文件保护； 2 内核模块保护）
static int kysec_whlist_load_inner(int kysec_type, const char *key,
				   struct object_list_inner *objects)
{
	int ret = 0;

	SLIST_INIT(objects);

	GVariant *parameters;
	if (NULL == key)
		parameters = g_variant_new("(is)", kysec_type, "");
	else
		parameters = g_variant_new("(is)", kysec_type, key);

	gchar *path, *hash, *type, *exectl, *filepro;
	gint rmmodpro, status;
	GVariantIter *iter;

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_LOAD, parameters, &err_code);
	if (NULL == result)
		return err_code;

	g_variant_get(g_variant_get_child_value(result, 0), "a(sssssii)", &iter);
	if (g_variant_iter_n_children(iter) == 0) {		
		goto out;
	}

	while (g_variant_iter_loop(iter, "(sssssii)", &path, &hash, &type, &exectl, &filepro, &rmmodpro, &status)) {
		if (0 != create_object_list_entry_inner(objects, path, hash, type, exectl, filepro, rmmodpro, status)) {
			ret = -3;
			break;
		}
	}

out:
	g_variant_unref(result);
	g_variant_iter_free(iter);

	return ret;
}

static char *get_package_name(char *debname)
{
	char *package_name = NULL;
	char *q = NULL;
	char *tmp_name = NULL;
	char *debname_tmp = NULL;

	package_name = malloc(NAME_MAX);
	if (NULL == package_name) {
		kysec_whlist_do_syslog("Failed to get package name from deb [%s]!\n", debname);
		return NULL;
	}
	memset(package_name, 0, NAME_MAX);

	debname_tmp = strndup(debname, strlen(debname));
	if (debname_tmp == NULL) {
		kysec_whlist_do_syslog("Failed to strndup for [%s]!\n", debname);
		free(package_name);
		return NULL;
	}

	q = strrchr(debname_tmp, '/');
	if (q == NULL) {
		q = strchr(debname_tmp, '_');
		if (NULL == q)
			strncpy(package_name, debname_tmp, strlen(debname_tmp));
		else {
			*q = '\0';
			strncpy(package_name, debname_tmp, strlen(debname_tmp));
		}
		goto out;
	}
	tmp_name = q + 1;
	*q = '\0';
	q = strchr(tmp_name, '_');
	if (NULL == q)
		strncpy(package_name, tmp_name, strlen(tmp_name));
	else {
		*q = '\0';
		strncpy(package_name, tmp_name, strlen(tmp_name));
	}

out:
	if (debname_tmp)
		free(debname_tmp);
	//printf("package_name='%s'\n", package_name);
	return package_name;
}

static int path_is_dir(const char *path)
{
	struct stat sb;

	if (!lstat(path, &sb) && S_ISDIR(sb.st_mode))
		return 1;

	return 0;
}

static char* get_full_path(char* fpath)
{
	char* full_path = NULL;
	char real_path[PATH_MAX] = {0};	

	if (access(fpath, F_OK) != 0)	{
		full_path = strdup(fpath);
		return full_path;
	}

	if (path_is_dir(fpath))	{
		realpath(fpath, real_path);
		full_path = strdup(real_path);
		return full_path;
	}

	char path[PATH_MAX] = {0};
	strncpy(path, fpath, sizeof(path));
	char* p = strrchr(path, '/');
	if (p != NULL)	{
		char* file_name = p + 1;
		*p = '\0';

		realpath(path, real_path);

		char apath[PATH_MAX] = {0};
		snprintf(apath, sizeof(apath), "%s/%s", real_path, file_name);
		full_path = strdup(apath);
	}
	else	{
		char pwd[PATH_MAX] = {0};
		getcwd(pwd, sizeof(pwd));
		char apath[PATH_MAX] = {0};
		snprintf(apath, sizeof(apath), "%s/%s", pwd, fpath);
		full_path = strdup(apath);
	}
	return full_path;
}

static void process_kernel_hash_by_paths(int type, int kysec_type, char *paths[], int count)
{
	struct object_list_inner list;
	struct object_entry_inner *item = NULL;
	int retval = 0;
	int mesg_type;
	retval = kysec_whlist_load_inner(kysec_type, NULL, &list);
	if (retval != 0) {
		kysec_whlist_do_syslog("load whlist failed\n");
		return;
	}

	for (int i = 0; i < count; i++) {
		if (paths[i] == NULL)
			continue;
		
		char* fpath = get_full_path(paths[i]);
		SLIST_FOREACH(item, &list, entries) {
			if (strcmp(fpath, item->object.fpath) == 0) {
				if (type)
					mesg_type = LABEL_ADD;
				else
					mesg_type = LABEL_DEL;
				retval = kysec_hash_communicate_with_kernel(mesg_type, item->object.hash, kysec_convert_exectl_label_to_label_type(item->object.exectl));
				if (retval != 0)
					kysec_whlist_do_syslog("process kernel hash by paths failed\n");
				break;
			}
		}
		free(fpath);
	}
	kysec_whlist_objects_inner_free(&list);
}

static void process_kernel_hash_by_dir(int type, int kysec_type, char *obj_type, char *dpath)
{
	struct object_list_inner list;
	struct object_entry_inner *item = NULL;
	int retval = 0;
	int mesg_type;

	if (dpath == NULL)
		return;

	char *dir = get_full_path(dpath);
	retval = kysec_whlist_load_inner(kysec_type, dir, &list);
	if (retval != 0) {
		free(dir);
		kysec_whlist_do_syslog("load whlist failed\n");
		return;
	}

	SLIST_FOREACH(item, &list, entries) {
		if (strncmp(dir, item->object.fpath, strlen(dir)) != 0)
			continue;

		if (obj_type != NULL && strcmp(obj_type, item->object.type) != 0)
			continue;

		if (type)
			mesg_type = LABEL_ADD;
		else
			mesg_type = LABEL_DEL;
		retval = kysec_hash_communicate_with_kernel(mesg_type, item->object.hash, kysec_convert_exectl_label_to_label_type(item->object.exectl));
		if (retval != 0)
			kysec_whlist_do_syslog("process kernel hash by dir failed\n");
		break;
	}

	kysec_whlist_objects_inner_free(&list);
	free(dir);
}

static void process_kernel_hash_by_deb(int type, const char *deb)
{
	struct object_list_inner exectl_list;
	struct object_entry_inner *exectl_item = NULL;
	struct object_list_inner kmodpro_list;
	struct object_entry_inner *kmodpro_item = NULL;
	int retval = 0;
	char cmd[1024] = {0};
	int mesg_type;

	char *package_name = get_package_name((char *)deb);
	if (NULL == package_name) {
		kysec_whlist_do_syslog("get package name failed, deb = %s\n", deb);
		return;
	}

	retval = kysec_whlist_load_inner(0, NULL, &exectl_list);
	if (retval != 0)
		kysec_whlist_do_syslog("load exectl list failed\n");

	retval = kysec_whlist_load_inner(2, NULL, &kmodpro_list);
	if (retval != 0)
		kysec_whlist_do_syslog("load kmodpro list failed\n");

	FILE *pp = NULL;
	char *line = NULL;
	size_t len = 0;
	ssize_t read;
	char *q = NULL;

	memset(cmd, 0, sizeof(cmd));
	snprintf(cmd, sizeof(cmd), "dpkg-query -L %s", package_name);
	pp = popen(cmd, "r");
	if (NULL == pp) {
		kysec_whlist_do_syslog("Failed to exec dpkg-query!\n");
		goto out;
	}

	while ((read = getline(&line, &len, pp)) != -1) {
		q = strrchr(line, '\n');
		*q = '\0';
		if (strstr(line, TMP_DIR)) /* tmp dir, skip */
			continue;

		if (path_is_dir(line))
			continue;

		retval = -1;
		SLIST_FOREACH(exectl_item, &exectl_list, entries) {
			if (strcmp(line, exectl_item->object.fpath) == 0) {
				if (type)
					mesg_type = LABEL_ADD;
				else
					mesg_type = LABEL_DEL;

				retval = kysec_hash_communicate_with_kernel(mesg_type, exectl_item->object.hash, kysec_convert_exectl_label_to_label_type(exectl_item->object.exectl));
				if (retval != 0)
					kysec_whlist_do_syslog("process kernel hash by deb failed\n");
				break;
			}
		}

		if (retval == 0)
			continue;

		SLIST_FOREACH(kmodpro_item, &kmodpro_list, entries) {
			if (strcmp(line, kmodpro_item->object.fpath) == 0) {
				if (type)
					mesg_type = LABEL_ADD;
				else
					mesg_type = LABEL_DEL;

				retval = kysec_hash_communicate_with_kernel(mesg_type, kmodpro_item->object.hash, kysec_convert_exectl_label_to_label_type(kmodpro_item->object.exectl));
				if (retval != 0)
					kysec_whlist_do_syslog("add hash to kernel failed\n");
				break;
			}
		}
	}

	free(line);
	pclose(pp);

out:
	if (package_name)
		free(package_name);

	kysec_whlist_objects_inner_free(&exectl_list);
	kysec_whlist_objects_inner_free(&kmodpro_list);
}

static void process_kernel_kmod_by_paths(int type, char *paths[], int count)
{
	char *kmod_name = NULL;
	char cmd[1024] = {0};
	int retval = 0;
	for (int i = 0; i < count; i++) {
		kmod_name = get_kmod_name_from_ko(paths[i]);
		if (NULL == kmod_name)
			continue;

		memset(cmd, 0x00, sizeof(cmd));
		if (type)
			snprintf(cmd, sizeof(cmd), "add %s", kmod_name);
		else
			snprintf(cmd, sizeof(cmd), "del %s", kmod_name);

		retval = kysec_kmod_communicate_with_kernel(cmd);
		if (retval != 0)
			kysec_whlist_do_syslog("process kernel kmod by paths failed\n");

		free(kmod_name);
		kmod_name = NULL;
	}
}

static void process_kernel_kmod_by_deb(int type, const char *deb)
{
	struct object_list_inner kmodpro_list;
	struct object_entry_inner *kmodpro_item = NULL;
	int retval = 0;
	char cmd[1024] = {0};
	char *kmod_name = NULL;

	char *package_name = get_package_name((char *)deb);
	if (NULL == package_name) {
		kysec_whlist_do_syslog("get package name failed, deb = %s\n", deb);
		return;
	}

	retval = kysec_whlist_load_inner(2, NULL, &kmodpro_list);
	if (retval != 0)
		kysec_whlist_do_syslog("load kmodpro list failed\n");

	FILE *pp = NULL;
	char *line = NULL;
	size_t len = 0;
	ssize_t read;
	char *q = NULL;

	memset(cmd, 0, sizeof(cmd));
	snprintf(cmd, sizeof(cmd), "dpkg-query -L %s", package_name);
	pp = popen(cmd, "r");
	if (NULL == pp) {
		kysec_whlist_do_syslog("Failed to exec dpkg-query!\n");
		goto out;
	}

	while ((read = getline(&line, &len, pp)) != -1) {
		q = strrchr(line, '\n');
		*q = '\0';
		if (strstr(line, TMP_DIR)) /* tmp dir, skip */
			continue;

		if (path_is_dir(line))
			continue;

		retval = -1;
		SLIST_FOREACH(kmodpro_item, &kmodpro_list, entries) {
			if (strcmp(line, kmodpro_item->object.fpath) == 0) {
				kmod_name = get_kmod_name_from_ko(line);
				if (NULL == kmod_name)
					break;

				memset(cmd, 0x00, sizeof(cmd));
				if (type)
					snprintf(cmd, sizeof(cmd), "add %s", kmod_name);
				else
					snprintf(cmd, sizeof(cmd), "del %s", kmod_name);

				retval = kysec_kmod_communicate_with_kernel(cmd);
				if (retval != 0)
					kysec_whlist_do_syslog("process kernel kmod by deb failed\n");

				free(kmod_name);
				kmod_name = NULL;
				break;
			}
		}
	}

	free(line);
	pclose(pp);

out:
	if (package_name)
		free(package_name);

	kysec_whlist_objects_inner_free(&kmodpro_list);
}

static void process_kernel_kmod_by_dir(int type, char *obj_type, char *dpath)
{
	struct object_list_inner list;
	struct object_entry_inner *item = NULL;
	int retval = 0;
	char cmd[1024] = {0};
	char *kmod_name = NULL;

	retval = kysec_whlist_load_inner(2, dpath, &list);
	if (retval != 0) {
		kysec_whlist_do_syslog("load whlist failed\n");
		return;
	}

	SLIST_FOREACH(item, &list, entries) {
		if (strncmp(dpath, item->object.fpath, strlen(dpath)) != 0)
			continue;

		if (obj_type != NULL && strcmp(obj_type, item->object.type) != 0)
			continue;

		kmod_name = get_kmod_name_from_ko(item->object.fpath);
		if (NULL == kmod_name)
			continue;

		memset(cmd, 0x00, sizeof(cmd));
		if (type)
			snprintf(cmd, sizeof(cmd), "add %s", kmod_name);
		else
			snprintf(cmd, sizeof(cmd), "del %s", kmod_name);
		retval = kysec_kmod_communicate_with_kernel(cmd);
		if (retval != 0)
			kysec_whlist_do_syslog("process kernel kmod by dir failed\n");

		free(kmod_name);
		kmod_name = NULL;
	}

	kysec_whlist_objects_inner_free(&list);
}

int kysec_whlist_add(int kysec_type, char *path)
{
	int ret = 0;

	if (path == NULL) {
		kysec_whlist_do_syslog("%s: the path is NULL\n", __func__);
		snprintf(err_msg, sizeof(err_msg), "%s: the paths is NULL\n", __func__);
		return -1;
	}

	char **paths = (char **)malloc(sizeof(char *) * 1);
	paths[0] = strdup(path);

	ret = kysec_whlist_adds(kysec_type, paths, 1);

	free(paths[0]);
	free(paths);

	return ret;
}

int kysec_whlist_adds(int kysec_type, char *paths[], int count)
{
	int ret = 0;

	if (paths == NULL || count == 0) {
		kysec_whlist_do_syslog("%s: the paths is NULL\n", __func__);
		snprintf(err_msg, sizeof(err_msg), "%s: the paths is NULL\n", __func__);
		return -1;
	}

	if (kysec_type != 1) {
		//非文件保护，先删除数据库中已有hash值
		process_kernel_hash_by_paths(0, kysec_type, paths, count);
	}

	GVariant *g_kysec_type = g_variant_new("i", kysec_type);

	GVariantBuilder *g_builder_parameter = g_variant_builder_new(G_VARIANT_TYPE_TUPLE);
	GVariantBuilder *g_builder_file_list = g_variant_builder_new(G_VARIANT_TYPE("as"));
	for (int i = 0; i < count; i++)	{
		if (paths[i] == NULL)
			continue;
		char *fpath = get_full_path(paths[i]);
		g_variant_builder_add(g_builder_file_list, "s", fpath);
		free(fpath);
	}

	GVariant *g_file_list = g_variant_builder_end(g_builder_file_list);

	g_variant_builder_add_value(g_builder_parameter, g_kysec_type);
	g_variant_builder_add_value(g_builder_parameter, g_file_list);

	GVariant *parameters = g_variant_builder_end(g_builder_parameter);

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_ADD, parameters, &err_code);
	if (NULL == result) {
		ret = err_code;
		goto out;
	}

	ret = g_variant_get_int32(g_variant_get_child_value(result, 0));
	g_variant_unref(result);

	if (kysec_type != 1) {
		//非文件保护，再将hash值写入数据库
		process_kernel_hash_by_paths(1, kysec_type, paths, count);
	}

out:
	g_variant_builder_unref(g_builder_file_list);
	g_variant_builder_unref(g_builder_parameter);
	return ret;
}

int kysec_whlist_dels(int kysec_type, char *paths[], int count)
{
	int ret = -1;

	if (paths == NULL || count == 0) {
		kysec_whlist_do_syslog("%s: the paths is NULL\n", __func__);
		snprintf(err_msg, sizeof(err_msg), "%s: the paths is NULL\n", __func__);
		return -1;
	}

	if (kysec_type != 1)
		process_kernel_hash_by_paths(0, kysec_type, paths, count);	//非文件保护，删除时，先删除内核中hash值，防止后续操作访问文件时，标记被重置

	GVariant *g_kysec_type = g_variant_new("i", kysec_type);

	GVariantBuilder *g_builder_parameter = g_variant_builder_new(G_VARIANT_TYPE_TUPLE);
	GVariantBuilder *g_builder_file_list = g_variant_builder_new(G_VARIANT_TYPE("as"));
	for (int i = 0; i < count; i++)	{
		if (paths[i] == NULL)
			continue;
		char *fpath = get_full_path(paths[i]);
		g_variant_builder_add(g_builder_file_list, "s", fpath);
		free(fpath);
	}

	GVariant *g_file_list = g_variant_builder_end(g_builder_file_list);

	g_variant_builder_add_value(g_builder_parameter, g_kysec_type);
	g_variant_builder_add_value(g_builder_parameter, g_file_list);

	GVariant *parameters = g_variant_builder_end(g_builder_parameter);

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_DEL, parameters, &err_code);
	if (NULL == result) {		
		ret = err_code;
		goto out;
	}

	ret = g_variant_get_int32(g_variant_get_child_value(result, 0));
	g_variant_unref(result);	

	// if (ret == 0 && kysec_type == 2)
	// 	process_kernel_kmod_by_paths(0, paths, count);

out:
	g_variant_builder_unref(g_builder_file_list);
	g_variant_builder_unref(g_builder_parameter);

	if (ret != 0 && kysec_type != 1)
		process_kernel_hash_by_paths(1, kysec_type, paths, count);		//非文件保护，若删除失败，重新设置内核中文件哈希值

	return ret;
}

int kysec_whlist_updates(int kysec_type, char *paths[], int count)
{
	int ret = -1;

	if (paths == NULL || count == 0) {
		kysec_whlist_do_syslog("%s: the paths is NULL\n", __func__);
		snprintf(err_msg, sizeof(err_msg), "%s: the paths is NULL\n", __func__);
		return -1;
	}

	if (kysec_type != 1)
		process_kernel_hash_by_paths(0, kysec_type, paths, count);	//非文件保护，更新时，先删除内核中hash值，防止后续操作访问文件时，标记被重置

	GVariant *g_kysec_type = g_variant_new("i", kysec_type);

	GVariantBuilder *g_builder_parameter = g_variant_builder_new(G_VARIANT_TYPE_TUPLE);
	GVariantBuilder *g_builder_file_list = g_variant_builder_new(G_VARIANT_TYPE("as"));
	for (int i = 0; i < count; i++)	{
		if (paths[i] == NULL)
			continue;
		char *fpath = get_full_path(paths[i]);
		g_variant_builder_add(g_builder_file_list, "s", fpath);
		free(fpath);
	}

	GVariant *g_file_list = g_variant_builder_end(g_builder_file_list);

	g_variant_builder_add_value(g_builder_parameter, g_kysec_type);
	g_variant_builder_add_value(g_builder_parameter, g_file_list);

	GVariant *parameters = g_variant_builder_end(g_builder_parameter);

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_UPDATE, parameters, &err_code);
	if (NULL == result) {
		ret = err_code;
		goto out;
	}

	ret = g_variant_get_int32(g_variant_get_child_value(result, 0));
	g_variant_unref(result);

out:
	g_variant_builder_unref(g_builder_file_list);
	g_variant_builder_unref(g_builder_parameter);

	if (kysec_type != 1)
		process_kernel_hash_by_paths(1, kysec_type, paths, count);	//不管更新是否成功，将数据中哈希值写入内核

	return ret;
}

int kysec_whlist_update_rmmodpros(int status, char *paths[], int count)
{
	int ret = -1;
	char *file_paths[count];
	for(int i=0;i<count;i++){
		file_paths[i]=(char*)malloc(128);
		memset(file_paths[i], 0, 128);
		memcpy(file_paths[i], paths[i], 128);
	}
	if (file_paths == NULL || count == 0) {
		kysec_whlist_do_syslog("%s: the paths is NULL\n", __func__);
		snprintf(err_msg, sizeof(err_msg), "%s: the paths is NULL\n", __func__);
		return -1;
	}

	GVariant *g_kysec_type = g_variant_new("i", status);

	GVariantBuilder *g_builder_parameter = g_variant_builder_new(G_VARIANT_TYPE_TUPLE);
	GVariantBuilder *g_builder_file_list = g_variant_builder_new(G_VARIANT_TYPE("as"));
	for (int i = 0; i < count; i++)	{
		char *fpath = get_full_path(file_paths[i]);
		g_variant_builder_add(g_builder_file_list, "s", fpath);
		free(fpath);
	}

	GVariant *g_file_list = g_variant_builder_end(g_builder_file_list);

	g_variant_builder_add_value(g_builder_parameter, g_kysec_type);
	g_variant_builder_add_value(g_builder_parameter, g_file_list);

	GVariant *parameters = g_variant_builder_end(g_builder_parameter);

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_UPDATE_RMMODPRO, parameters, &err_code);
	if (NULL == result) {
		ret = err_code;
		goto out;
	}

	ret = g_variant_get_int32(g_variant_get_child_value(result, 0));
	g_variant_unref(result);

	if (ret == 0){
		for (int i = 0; i < count; i++)	{
			if (NULL != strstr(file_paths[i], ".xz")) {
				char kmod_cmd[256]={0};
				sprintf(kmod_cmd, "mkdir /tmp/kmod_dir;cp %s /tmp/kmod_dir/;xz -d /tmp/kmod_dir/*;realpath /tmp/kmod_dir/* > /tmp/kmod_dir/filename", file_paths[i]);
				system(kmod_cmd);
				FILE *fp = fopen("/tmp/kmod_dir/filename","r");
				memset(file_paths[i],0,128);
				fgets(file_paths[i],128,fp);
				file_paths[i][strlen(file_paths[i])-1] = 0;
				fclose(fp);
			}
		}
		process_kernel_kmod_by_paths(status, file_paths, count);
	}

out:

	g_variant_builder_unref(g_builder_file_list);
	g_variant_builder_unref(g_builder_parameter);
	system("rm -rf /tmp/kmod_dir");
	return ret;
}

int kysec_whlist_load(int kysec_type, const char *key, struct object_list *objects)
{
	int ret = 0;

	SLIST_INIT(objects);

	GVariant *parameters = NULL;

	if (NULL == key)
		parameters = g_variant_new("(is)", kysec_type, "");
	else
		parameters = g_variant_new("(is)", kysec_type, key);

	gchar *path, *hash, *type, *exectl, *filepro;
	gint rmmodpro, status;
	GVariantIter *iter;

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_LOAD, parameters, &err_code);
	if (NULL == result)
		return err_code;

	g_variant_get(g_variant_get_child_value(result, 0), "a(sssssii)", &iter);
	if (g_variant_iter_n_children(iter) == 0) {		
		goto out;
	}

	while (g_variant_iter_loop(iter, "(sssssii)", &path, &hash, &type, &exectl, &filepro, &rmmodpro, &status)) {
		if (0 != create_object_list_entry(objects, path, hash, type, exectl, filepro, rmmodpro, status)) {
			ret = -3;
			break;
		}
	}

out:

	g_variant_unref(result);
	g_variant_iter_free(iter);

	return ret;
}

int kysec_whlist_import(int kysec_type, char *file)
{
	int ret = -1;

	if (file == NULL) {
		kysec_whlist_do_syslog("%s: the import file is NULL\n", __func__);
		snprintf(err_msg, sizeof(err_msg), "%s: the import file is NULL\n", __func__);
		return -1;
	}

	GVariant *parameters = g_variant_new("(is)", kysec_type, file);

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_IMPORT, parameters, &err_code);
	if (NULL == result) {
		ret = err_code;
		goto out;
	}

	ret = g_variant_get_int32(g_variant_get_child_value(result, 0));

	g_variant_unref(result);

out:
	return ret;
}

void kysec_whlist_objects_free(struct object_list *objects)
{
	struct object_entry *entry;

	if (!objects)
		return;

	entry = SLIST_FIRST(objects);
	for (; entry != NULL; entry = SLIST_FIRST(objects)) {
		SLIST_REMOVE_HEAD(objects, entries);
		if (entry->object.fpath) {
			free(entry->object.fpath);
			entry->object.fpath = NULL;
		}

		if (entry->object.hash) {
			free(entry->object.hash);
			entry->object.hash = NULL;
		}

		if (entry) {
			free(entry);
			entry = NULL;
		}
	}

	return;
}

char *kysec_whlist_last_err()
{
	return err_msg;
}

int kysec_whlist_export(int kysec_type, char *file)
{
	int ret = -1;

	if (file == NULL) {
		kysec_whlist_do_syslog("%s: the import file is NULL\n", __func__);
		snprintf(err_msg, sizeof(err_msg), "%s: the import file is NULL\n", __func__);
		return -1;
	}

	GVariant *parameters = g_variant_new("(is)", kysec_type, file);

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_EXPORT, parameters, &err_code);
	if (NULL == result) {
		ret = err_code;
		goto out;
	}

	ret = g_variant_get_int32(g_variant_get_child_value(result, 0));

	g_variant_unref(result);

out:

	return ret;
}

int kysec_whlist_del(int kysec_type, char *path)
{
	int ret = 0;

	if (path == NULL) {
		kysec_whlist_do_syslog("%s: the path is NULL\n", __func__);
		snprintf(err_msg, sizeof(err_msg), "%s: the paths is NULL\n", __func__);
		return -1;
	}

	char **paths = (char **)malloc(sizeof(char *) * 1);
	paths[0] = strdup(path);

	ret = kysec_whlist_dels(kysec_type, paths, 1);

	free(paths[0]);
	free(paths);

	return ret;
}

int kysec_whlist_update(int kysec_type, char *path)
{
	int ret = 0;

	if (path == NULL) {
		kysec_whlist_do_syslog("%s: the path is NULL\n", __func__);
		snprintf(err_msg, sizeof(err_msg), "%s: the paths is NULL\n", __func__);
		return -1;
	}

	char **paths = (char **)malloc(sizeof(char *) * 1);
	paths[0] = strdup(path);

	ret = kysec_whlist_updates(kysec_type, paths, 1);

	free(paths[0]);
	free(paths);
	return ret;
}

int kysec_whlist_update_rmmodpro(int status, char *path)
{
	int ret = 0;

	if (path == NULL) {
		kysec_whlist_do_syslog("%s: the path is NULL\n", __func__);
		snprintf(err_msg, sizeof(err_msg), "%s: the paths is NULL\n", __func__);
		return -1;
	}

	char **paths = (char **)malloc(sizeof(char *) * 1);
	paths[0] = strdup(path);

	ret = kysec_whlist_update_rmmodpros(status, paths, 1);

	free(paths[0]);
	free(paths);

	return ret;
}

int kysec_whlist_get_count_by_obj_type(int kysec_type, const char *obj_type, const char *key)
{
	int ret = 0;

	GVariant *parameters = NULL;

	if (NULL == key)
		parameters = g_variant_new("(iss)", kysec_type, obj_type, "");
	else
		parameters = g_variant_new("(iss)", kysec_type, obj_type, key);

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_GET_COUNT_BY_OBJECT_TYPE, parameters, &err_code);
	if (NULL == result)
		return 0;

	ret = g_variant_get_int32(g_variant_get_child_value(result, 0));
	g_variant_unref(result);

	return ret;
}

int kysec_whlist_load_by_obj_type(int kysec_type, const char *obj_type, const char *key, struct object_list *objects)
{
	int ret = 0;

	SLIST_INIT(objects);

	GVariant *parameters = NULL;

	if (NULL == key)
		parameters = g_variant_new("(iss)", kysec_type, obj_type, "");
	else
		parameters = g_variant_new("(iss)", kysec_type, obj_type, key);

	gchar *path, *hash, *type, *exectl, *filepro;
	gint rmmodpro, status;
	GVariantIter *iter;

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_LOAD_BY_OBJECT_TYPE, parameters, &err_code);
	if (NULL == result)
		return err_code;

	g_variant_get(g_variant_get_child_value(result, 0), "a(sssssii)", &iter);
	if (g_variant_iter_n_children(iter) == 0) {		
		goto out;
	}

	while (g_variant_iter_loop(iter, "(sssssii)", &path, &hash, &type, &exectl, &filepro, &rmmodpro, &status)) {
		if (0 != create_object_list_entry(objects, path, hash, type, exectl, filepro, rmmodpro, status)) {
			ret = -3;
			break;
		}
	}

out:
	g_variant_unref(result);
	g_variant_iter_free(iter);

	return ret;
}

int kysec_whlist_load_by_path(const char *path, struct db_object *object)
{
	int ret = 0;

	GVariant *parameters = NULL;

	if (NULL == path)
		return -1;

	char *file_path = get_full_path(path);
	parameters = g_variant_new("(s)", file_path);
	free(file_path);

	gchar *fpath = NULL, *hash = NULL, *type = NULL, *exectl = NULL, *filepro = NULL;
	gint rmmodpro, status;

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_LOAD_BY_PATH, parameters, &err_code);
	if (NULL == result)
		return err_code;

	g_variant_get(g_variant_get_child_value(result, 0), "(sssssii)", &fpath, &hash, &type, &exectl, &filepro, &rmmodpro, &status);

	if (fpath)
		object->fpath = strndup(fpath, strlen(fpath));
	else
		object->fpath = NULL;

	if (hash)
		object->hash = strndup(hash, strlen(hash));
	else
		object->hash = NULL;
	object->type = parse_kysec_file_type(type);
	object->exectl = parse_exectl_type(exectl);
	object->filepro = parse_protect_type(filepro);
	object->rmmodpro = rmmodpro;
	object->status = status;

out:
	g_variant_unref(result);

	return ret;
}

int kysec_whlist_add_by_deb(const char *deb)
{
	int ret = 0;

	if (NULL == deb)
		return -1;

	process_kernel_hash_by_deb(0, deb);

	GVariant *parameters = NULL;

	char *fpath = get_full_path(deb);
	parameters = g_variant_new("(s)", fpath);
	free(fpath);

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_ADD_BY_DEB, parameters, &err_code);
	if (NULL == result)
		return err_code;

	ret = g_variant_get_int32(g_variant_get_child_value(result, 0));
	g_variant_unref(result);

	process_kernel_hash_by_deb(1, deb);

	return ret;
}

int kysec_whlist_update_by_deb(const char *deb)
{
	int ret = 0;

	if (NULL == deb)
		return -1;

	process_kernel_hash_by_deb(0, deb);		//删除内核中hash表，防止后续文件操作导致文件标记被还原

	GVariant *parameters = NULL;

	char *fpath = get_full_path(deb);
	parameters = g_variant_new("(s)", fpath);
	free(fpath);
	
	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_UPDATE_BY_DEB, parameters, &err_code);
	if (NULL == result)
		return err_code;

	ret = g_variant_get_int32(g_variant_get_child_value(result, 0));
	g_variant_unref(result);

	process_kernel_hash_by_deb(1, deb);	//将hash值重新写入内核

	return ret;
}

int kysec_whlist_del_by_deb(const char *deb)
{
	int ret = 0;

	if (NULL == deb)
		return -1;

	process_kernel_hash_by_deb(0, deb);		//删除内核中hash表，防止后续文件操作导致文件标记被还原

	GVariant *parameters = NULL;
	char *fpath = get_full_path(deb);
	parameters = g_variant_new("(s)", fpath);
	free(fpath);

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_DEL_BY_DEB, parameters, &err_code);
	if (NULL == result)	{
		ret = err_code;
		goto out;
	}

	ret = g_variant_get_int32(g_variant_get_child_value(result, 0));
	g_variant_unref(result);

	// if (ret == 0)
	// 	process_kernel_kmod_by_deb(0, deb);

out:
	if (ret != 0)
		process_kernel_hash_by_deb(1, deb);		//将hash值重新写入内核

	return ret;
}

int kysec_whlist_add_by_dir(int kysec_type, const char *obj_type, const char *dpath)
{
	int ret = 0;

	if (NULL == dpath)
		return -1;

	if (kysec_type != 1)
		process_kernel_hash_by_dir(0, kysec_type, (char *)obj_type, (char *)dpath);

	GVariant *parameters = NULL;

	char *dir = get_full_path(dpath);
	if (NULL != obj_type)
		parameters = g_variant_new("(iss)", kysec_type, obj_type, dir);
	else
		parameters = g_variant_new("(iss)", kysec_type, "", dir);
	free(dir);

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_ADD_BY_DIR, parameters, &err_code);
	if (NULL == result)
		return err_code;

	ret = g_variant_get_int32(g_variant_get_child_value(result, 0));
	g_variant_unref(result);

	if (kysec_type != 1)
		process_kernel_hash_by_dir(1, kysec_type, (char *)obj_type, (char *)dpath);

	return ret;
}

int kysec_whlist_update_by_dir(int kysec_type, const char *obj_type, const char *dpath)
{
	int ret = 0;

	if (NULL == dpath)
		return -1;

	if (kysec_type != 1)
		process_kernel_hash_by_dir(0, kysec_type, (char *)obj_type, (char *)dpath);

	struct object_list_inner list;
	struct object_entry_inner *item = NULL;
	int retval = 0;
	char cmd[1024] = {0};

	if (dpath == NULL)
		return -1;

	GVariant *parameters = NULL;

	char *dir = get_full_path(dpath);
	if (NULL != obj_type)
		parameters = g_variant_new("(iss)", kysec_type, obj_type, dir);
	else
		parameters = g_variant_new("(iss)", kysec_type, "", dir);
	free(dir);

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_UPDATE_BY_DIR, parameters, &err_code);
	if (NULL == result)
		return err_code;

	ret = g_variant_get_int32(g_variant_get_child_value(result, 0));
	g_variant_unref(result);

	if (kysec_type != 1)
		process_kernel_hash_by_dir(1, kysec_type, (char *)obj_type, (char *)dpath);

	return ret;
}

int kysec_whlist_del_by_dir(int kysec_type, const char *obj_type, const char *dpath)
{
	int ret = 0;

	if (NULL == dpath)
		return -1;

	process_kernel_hash_by_dir(0, kysec_type, (char *)obj_type, (char *)dpath);

	struct object_list_inner list;
	struct object_entry_inner *item = NULL;
	int retval = 0;
	char cmd[1024] = {0};

	if (dpath == NULL)
		return -1;

	GVariant *parameters = NULL;

	char *dir = get_full_path(dpath);
	if (NULL != obj_type)
		parameters = g_variant_new("(iss)", kysec_type, obj_type, dir);
	else
		parameters = g_variant_new("(iss)", kysec_type, "", dir);
	free(dir);	

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_DEL_BY_DIR, parameters, &err_code);
	if (NULL == result)
	{
		ret = err_code;
		goto out;		
	}

	ret = g_variant_get_int32(g_variant_get_child_value(result, 0));
	g_variant_unref(result);

	// if (ret == 0 && kysec_type == 2)
	// 	process_kernel_kmod_by_dir(0, (char *)obj_type, (char *)dpath);

out:
	if (ret != 0 && kysec_type != 1)
		process_kernel_hash_by_dir(1, kysec_type, (char *)obj_type, (char *)dpath);

	return ret;
}

char *kysec_whlist_get_hash_by_path(char *fpath)
{
	if (fpath == NULL)
		return NULL;

	char *file_path = get_full_path(fpath);
	GVariant *parameters = g_variant_new("(s)", file_path);
	free(file_path);

	gchar *ghash = NULL;

	int err_code;
	GVariant *result = kysec_whlist_process_dbus_method(
		KYSEC_WHLIST_METHOD_GET_HASH_BY_PATH, parameters, &err_code);
	if (NULL == result)
		return NULL;

	ghash = g_variant_get_string(g_variant_get_child_value(result, 0), NULL);
	char *hash = strdup(ghash);

	g_variant_unref(result);
	return hash;
}

int kysec_whlist_exectl_add(char* path)
{
	int ret = 0;

	if (path == NULL) {
		kysec_whlist_do_syslog("%s: the path is NULL\n", __func__);
		snprintf(err_msg, sizeof(err_msg), "%s: the paths is NULL\n", __func__);
		return -2;
	}

	char **paths = (char **)malloc(sizeof(char *) * 1);
	paths[0] = strdup(path);

	int res = kysec_whlist_adds(0, paths, 1);
	if (res != 0) {
		if (res == 1 || res == 2)
			ret = 1;
		else if (res == 7)
			ret = -3;
		else
			ret = -1;		
	}

	free(paths[0]);
	free(paths);

	return ret;
}

int kysec_whlist_exectl_update(char* path)
{
	int ret = 0;

	if (path == NULL) {
		kysec_whlist_do_syslog("%s: the path is NULL\n", __func__);
		snprintf(err_msg, sizeof(err_msg), "%s: the paths is NULL\n", __func__);
		return -2;
	}

	char **paths = (char **)malloc(sizeof(char *) * 1);
	paths[0] = strdup(path);

	int res = kysec_whlist_updates(0, paths, 1);
	if (res != 0) {
		if (res == 1 || res == 2)
			ret = 1;
		else if (res == 7)
			ret = -3;
		else
			ret = -1;		
	}

	free(paths[0]);
	free(paths);

	return ret;
}

int kysec_whlist_exectl_remove(char* path)
{
	int ret = 0;

	if (path == NULL) {
		kysec_whlist_do_syslog("%s: the path is NULL\n", __func__);
		snprintf(err_msg, sizeof(err_msg), "%s: the paths is NULL\n", __func__);
		return -2;
	}

	char **paths = (char **)malloc(sizeof(char *) * 1);
	paths[0] = strdup(path);

	int res = kysec_whlist_dels(0, paths, 1);
	if (res != 0) {
		if (res == 1 || res == 2)
			ret = 1;
		else if (res == 7)
			ret = -3;
		else
			ret = -1;		
	}

	free(paths[0]);
	free(paths);

	return ret;
}