#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <openssl/evp.h>
#include <openssl/sha.h>
#include "kysecwhlist/sm3_hash.h"

#define SM3_DIGEST_LENGTH SHA256_DIGEST_LENGTH

char *get_hash_value(const char *filepath)
{
	int i = 0;
	unsigned char result[SM3_DIGEST_LENGTH] = {0};
	char *value = NULL;
	EVP_MD_CTX *ctx = NULL;
	const EVP_MD *md = NULL;
	FILE *fp = NULL;
	unsigned char content[1024];
	size_t rlen = 0;
	int md_len;

	fp = fopen(filepath, "rb");
	if (fp == NULL) {
		printf("Failed to open [%s]!\n", filepath);
		return NULL;
	}

	value = malloc(SM3_DIGEST_LENGTH * 2 + 1);
	if (NULL == value) {
		printf("failed to malloc!\n");
		goto out;
	}
	memset(value, 0, SM3_DIGEST_LENGTH * 2 + 1);
	memset(result, 0x0, sizeof(result));

	md = EVP_sm3();
	ctx = EVP_MD_CTX_new();
	if (!md || !ctx) {
		printf("Failed to get sm3 algo\n");
		free(value);
		value = NULL;
		goto out;
	}

	EVP_DigestInit_ex(ctx, md, NULL);
	do {
		memset(content, 0, sizeof(content));
		rlen = fread(content, 1, 1024, fp);
		EVP_DigestUpdate(ctx, content, rlen);
	} while (rlen == 1024);

	EVP_DigestUpdate(ctx, filepath, strlen(filepath));
	EVP_DigestFinal_ex(ctx, result, &md_len);
	EVP_MD_CTX_free(ctx);

	if (md_len != SM3_DIGEST_LENGTH) {
		printf("Failed to get sm3 value for %s\n", filepath);
		free(value);
		value = NULL;
		goto out;
	}

	for (i = 0; i < SM3_DIGEST_LENGTH; i++)
		snprintf(value + i*2, 3, "%02x", result[i]);
	value[SM3_DIGEST_LENGTH * 2] = '\0';

out:
	fclose(fp);

	return value;
}
