/*
 *  Kylin security execute control utils
 *
 *  Authors:  songchuanfei <songchuanfei@kylinos.cn>
 *  Authors:  huanfuxi <huanfuxi@kylinos.cn>
 *  Authors:  mengyuan <mengyuan@kylinos.cn>
 *
 *  Copyright (C) 2017,Tianjin KYLIN Information Technology Co., Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <ftw.h>
#include <kysec/libkysec.h>
#include <getopt.h>
#include <dirent.h>

int recursive = 0;
int sid_type = 0;

static int kysec_get_label_for_process(pid_t pid)
{
	int retval = 0;
	int err = -1;
	char *label = NULL;

	if (kysec_label_get_by_pid(pid, &label) == 0) {
		printf("%d: %s\n", pid, label);
		free(label);
	} else {
		//printf("%d: %s\n", pid, strerror(errno));
		retval = -1;
	}

	return retval;
}

static int kysec_get_label_for_file(char *path, int sid_type)
{
	char *label_str = NULL;
	char *tokens[3] = {NULL};
	char *buf = NULL;

	int i = 0, j = 0;
	int label_len;

	label_len = kysec_label_get(path, &label_str);
	if (label_len < 0)
		return -1;

	buf = malloc(label_len + 1);
	if (!buf) {
		free(label_str);
		return -ENOMEM;
	}
	tokens[0] = buf;

	for (i = 0, j = 1; i < label_len; i++) {
		if (*(label_str + i) == ':') {
			*(buf + i) = '\0';
			tokens[j++] = buf + i + 1;
		} else
			*(buf + i) = *(label_str + i);

		if (j > 3)
			break;
	}
	buf[label_len] = '\0';
	label_str[label_len] = '\0';

	switch (sid_type) {
	case 'e':
		printf("%s: %s\n", path, tokens[2]);
		break;
	case 'p':
		printf("%s: %s\n", path, tokens[1]);
		break;
	case 'i':
		printf("%s: %s\n", path, tokens[0]);
		break;
	case 'a':
		printf("%s: %s\n", path, label_str);
		break;
	}

	free(buf);
	free(label_str);
	return 0;
}

static int path_is_dir(const char *path)
{
	struct stat sb;

	if (!lstat(path, &sb) && S_ISDIR(sb.st_mode))
		return 1;

	// make kernel to check whether it's a directory
	return 0;
}

static int kysec_get_label_for_path_v2(char *path, int sid_type)
{
	if (kysec_is_disabled()) {
		errno = ENOTSUP;
		return -1;
	}

	struct stat sb;

	if (!lstat(path, &sb) && S_ISDIR(sb.st_mode)) {		
		if (!recursive) {
			errno = EISDIR;
			return -1;
		}

		DIR *dp;                      
    	struct dirent *entry;         
    	struct stat statbuf; 
		char fpath [2048] = {0};  
		int lenstr = 0;

    	if((dp = opendir(path)) == NULL) {
			errno = ENOENT;         	
        	return -1;  
    	}  

		lenstr = strlen(path);
		if ('/' == path[lenstr - 1])
			path[lenstr - 1] = '\0';
    
    	while((entry = readdir(dp)) != NULL) {
			memset(fpath, 0x00, sizeof(fpath));
			snprintf(fpath, sizeof(fpath), "%s/%s", path, entry->d_name);
  
        	lstat(fpath, &statbuf); 
        	if(entry->d_type == DT_DIR) {  
            	if (strcmp(".", entry->d_name) == 0 || strcmp("..", entry->d_name) == 0)  
              		continue;    
            	 
				kysec_get_label_for_file(fpath, sid_type);			
            	kysec_get_label_for_path_v2(fpath, sid_type);              // 递归调用自身，扫描下一级目录的内容  
        	}  
        	else  {
            	if (kysec_get_label_for_file(fpath, sid_type) < 0) {
					closedir(dp);
					return -1;
				}
			}   	  
        	
		} 
		closedir(dp);	
	}
	else
		return kysec_get_label_for_file(path, sid_type);	
}

static int kysec_get_label_for_path(char *path, int sid_type)
{
	if (kysec_is_disabled()) {
		errno = ENOTSUP;
		return -1;
	}

	struct stat sb;

	if (!lstat(path, &sb) && S_ISDIR(sb.st_mode)) {
		char *pathv[2] = {0};
		if (!recursive) {
			errno = EISDIR;
			return -1;
		}

		pathv[0] = path;
		pathv[1] = NULL;
		FTS *fts = fts_open(pathv, FTS_DEFAULT, NULL);
		FTSENT *ent = NULL;

		while (1) {
			ent = fts_read(fts);
			if (!ent) {
				if (errno != 0)
					continue;
				else
					break;
			}

			if (path_is_dir(ent->fts_path)) {
				//kysec_get_label_for_file(ent->fts_path, sid_type);     // 不再获取目录标记	
				continue;
			}

			if (kysec_get_label_for_file(ent->fts_path, sid_type) < 0) {
				fts_close(fts);
				return -1;
			}
		}

		fts_close(fts);
	} else
		return kysec_get_label_for_file(path, sid_type);
}

static int get_kysec_label_for_package(char *pkg_name, int sid_type)
{
	if (kysec_is_disabled()) {
		errno = ENOTSUP;
		return -1;
	}

	char command[4097], linebuf[1024], *path = NULL;

	sprintf(command, "dpkg -L %s >/tmp/dpkg-list 2>/dev/null", pkg_name);
	if (system(command)) {
		fprintf(stderr, "%s: package  not installed!\n", pkg_name);
		return -1;
	}

	FILE *fp = fopen("/tmp/dpkg-list", "r");
	if (!fp) {
		fprintf(stderr, "%s: package not installed!\n", pkg_name);
		return -1;
	}

	while (fgets(linebuf, 1024, fp)) {
		struct stat st;
		FILE *tmp = NULL;
		linebuf[strlen(linebuf)-1] = '\0';
		path = linebuf;
		stat(path, &st);

		if (S_ISDIR(st.st_mode) || !(tmp = fopen(path, "r")))
			continue;

		fclose(tmp);

		if (kysec_get_label_for_path(path, sid_type) < 0) {
			fprintf(stderr, "%s: %s\n", path, strerror(errno));
			return -1;
		}
	}
	fclose(fp);

	if (system("rm -f /tmp/dpkg-list")) {
		fprintf(stderr, "remove tmp file failed\n");
		return -1;
	}

	return 0;
}

static void usage(void)
{
	printf("Usage:kysec_get [ -r ] [ -R ] [ -n LABEL_NAME ] PATH1 PATH2 ...\n");
	printf("      kysec_get -p PID\n");
	printf("      kysec_get --package PACKAGE_NAME\n");
	printf("\t -r: get kysec label recursively for directories\n");
	printf("\t -n: get the specified label only, label_name can be exectl, protect or userid\n");
	printf("\t -p: get the label for process PID\n");
	printf("\t --package: get the label for a installed package\n");
	printf("\t -h: show help information.\n");
	exit(1);
}

int main(int argc, char **argv)
{
	int opt, index;
	char *path = NULL, *label_name = "all", *buff, *pkg_name = NULL;
	int option_index = 0;
	static struct option long_options[] = {
		{"package", required_argument, 0, 0},
		{0, 0, 0, 0}
	};

	while ((opt = getopt_long(argc, argv, "Rrpn:v:h", long_options, &option_index)) != -1) {
		switch (opt) {
		case 0:
			pkg_name = optarg;
			break;
		case 'r':
			recursive = 1;
			break;
		case 'n':
			label_name = optarg;
			break;
		case 'p':
			if (optind != argc-1) {
				usage();
				return -1;
			} else {
				pid_t pid;

				pid = strtol(argv[optind], &buff, 10);
				if (*buff != '\0') {
					fprintf(stderr, "%s: invaild process id\n", argv[optind]);
					return -1;
				}

				if (kysec_get_label_for_process(pid) < 0) {
					if (errno == ENOTSUP)
						fprintf(stderr, "Kysec is Disabled\n");
					else
						fprintf(stderr, "%d: %s\n", pid, strerror(errno));
					return -1;
				}
			}
			return 0;
		case 'h':
			usage();
			return 0;
		default:
			fprintf(stderr, "invalid arguments\n");
			usage();
			return -1;
		}
	}

	if (optind < 0)
		goto invalid;

	if (!strcmp( label_name, "exectl" )) {
		sid_type = 'e';
	} else if (!strcmp(label_name, "protect")) {
		sid_type = 'p';
	} else if (!strcmp(label_name, "userid")) {
		sid_type = 'i';
	} else if (!strcmp(label_name, "all")) {
		sid_type = 'a';
	} else {
		fprintf(stderr, "Unknown label name: %s\n", label_name);
		goto invalid;
	}

	if (pkg_name) {
		if (optind != argc) {
			fprintf(stderr, "invalid arguments\n");
			usage();
			return -1;
		} else {
			if (get_kysec_label_for_package(pkg_name, sid_type) < 0) {
				if (errno == ENOTSUP)
					fprintf(stderr, "Kysec is Disabled\n");
				return -1;
			}
			return 0;
		}
	}

	if (argc <= optind)
		goto invalid;

	for (int i = optind; i < argc; i++) {
		path=argv[i];

		if ('/' == path[strlen(path) - 1])
			path[strlen(path) - 1] = '\0';

		if (kysec_get_label_for_path(path, sid_type) < 0) {
			if (errno == ENOTSUP)
				fprintf(stderr, "Kysec is Disabled\n");
			else
				fprintf(stderr, "%s: %s\n", path, strerror(errno));

			if (optind == argc - 1)
				return -1;
			continue;
		}
	}

	return 0;

invalid:
	usage();
	return -1;
}
