#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <kysec/kmod_protect.h>

static void usage(void)
{
	printf("Usage:\n");
	printf("\t kysec_kmod [-a|-d|-n] FILE_PATH\n");
	printf("\t kysec_kmod [-h|--help]\n");
	printf("\t [-a]: add kmod rmmod pro\n");
	printf("\t [-d]: delete kmod rmmod pro");
	printf("\t [-n]: get kmod name from kmod file\n");
	printf("\t [-h|--help]: show help information.\n");
	exit(1);
}

int main(int argc, char *argv[])
{
	int c;
	int add = 0, delete = 0, name = 0;

	while (1) {
		int option_index;
		static struct option long_options[] = {
			{"add", required_argument, 0, 'a'},
			{"delete", required_argument, 0, 'd'},
			{"name", required_argument, 0, 'n'},
			{"help", no_argument, 0, 'h'},
			{0, 0, 0, 0}
		};

		c = getopt_long(argc, argv, "adnh", long_options, &option_index);
		if (c == -1)
			break;

		switch (c) {
		case 'a':
			add = 1;
			break;
		case 'd':
			delete = 1;
			break;
		case 'n':
			name = 1;
			break;
		case 'h':
			usage();
			exit(EXIT_SUCCESS);
		default:
			fprintf(stderr, "Invalid operations\n");
			usage();
			exit(EXIT_SUCCESS);
		}
	}

	if (optind != argc - 1) {
		fprintf(stderr, "Invalid path input\n");
		exit(EXIT_SUCCESS);
	}

	int ret = 0;
	if (add) {
		ret = add_to_kmod_protect(argv[optind]);
		if (0 != ret) {
			fprintf(stderr, "add kmod protect failed, ret = %d\n", ret);
			return 0;
		}
	}

	if (delete) {
		ret = remove_from_kmod_protect(argv[optind]);
		if (0 != ret) {
			fprintf(stderr, "delete kmod protect failed, ret = %d\n", ret);
			return 0;
		}
	}

	if (name) {
		char *kmod_name = get_kmod_name_from_ko(argv[optind]);

		if (NULL == kmod_name) {
			fprintf(stderr, "get kmod name from ko file failed\n");
			return 0;
		}

		printf("kmod_name: %s\n", kmod_name);
		free(kmod_name);
	}

	return 0;
}
